@@ -48,25 +48,9 @@ class PyRunner:
     self.register = NodeRegister()
     self.engine = ExecutionEngine()
     
-    self._init_params = {
-      'config_file'          : kwargs.get('config_file'),
-      'proc_file'            : kwargs.get('proc_file'),
-      'restart'              : kwargs.get('restart', False),
-      'cvar_list'            : [],
-      'exec_proc_name'       : None,
-      'exec_only_list'       : [],
-      'exec_disable_list'    : [],
-      'exec_from_id'         : None,
-      'exec_to_id'           : None
-    }
-    
-    # Lifecycle hooks
-    self._on_create_func = None
-    self._on_start_func = None
-    self._on_restart_func = None
-    self._on_success_func = None
-    self._on_fail_func = None
-    self._on_destroy_func = None
+    self.config['config_file'] = kwargs.get('config_file')
+    self.config['proc_file'] = kwargs.get('proc_file')
+    self.config['restart'] = kwargs.get('restart', False)
     
     self.parse_args()
     
@@ -109,18 +93,18 @@ class PyRunner:
   
   @property
   def config_file(self):
-    return self._init_params['config_file']
+    return self.config['config_file']
   @config_file.setter
   def config_file(self, value):
-    self._init_params['config_file'] = value
+    self.config['config_file'] = value
     return self
   
   @property
   def proc_file(self):
-    return self._init_params['proc_file']
+    return self.config['proc_file']
   @proc_file.setter
   def proc_file(self, value):
-    self._init_params['proc_file'] = value
+    self.config['proc_file'] = value
     return self
   
   @property
@@ -129,7 +113,7 @@ class PyRunner:
   
   @property
   def restart(self):
-    return self._init_params['restart']
+    return self.config['restart']
   
   def plugin_serde(self, obj):
     if not isinstance(obj, serde.SerDe): raise Exception('SerDe plugin must implement the SerDe interface')
@@ -139,59 +123,49 @@ class PyRunner:
     if not isinstance(obj, notification.Notification): raise Exception('Notification plugin must implement the Notification interface')
     self.notification = obj
   
-  # App lifecycle hooks/decorators
-  def on_create(self, func):
-    self._on_create_func = func
-  def on_start(self, func):
-    self._on_start_func = func
-  def on_restart(self, func):
-    self._on_restart_func = func
-  def on_success(self, func):
-    self._on_success_func = func
-  def on_fail(self, func):
-    self._on_fail_func = func
-  def on_destroy(self, func):
-    self._on_destroy_func = func
+  # Engine wiring
+  def on_create(self, func) : self.engine.on_create(func)
+  def on_start(self, func)  : self.engine.on_start(func)
+  def on_restart(self, func): self.engine.on_restart(func)
+  def on_success(self, func): self.engine.on_success(func)
+  def on_fail(self, func)   : self.engine.on_fail(func)
+  def on_destroy(self, func): self.engine.on_destroy(func)
+  
+  # NodeRegister wiring
+  def add_node(self, **kwargs)    : return self.register.add_node(**kwargs)
+  def exec_only(self, id_list)    : return self.register.exec_only(id_list)
+  def exec_to(self, id)           : return self.register.exec_to(id)
+  def exec_from(self, id)         : return self.register.exec_from(id)
+  def exec_disable(self, id_list) : return self.register.exec_disable(id_list)
   
   def prepare(self):
     # Initialize NodeRegister
-    if self._init_params['restart']:
+    if self.config['restart']:
       self.load_state()
-    elif self._init_params.get('proc_file'):
-      self.load_proc_file(self._init_params['proc_file'])
+    elif self.config['proc_file']:
+      self.load_proc_file(self.config['proc_file'])
     
     # Inject Context var overrides
-    for k,v in self._init_params['cvar_list']:
+    for k,v in self.config['cvar_list']:
       self.engine.context.set(k, v)
     
     # Modify NodeRegister
-    if self._init_params['exec_proc_name']:
-      self.exec_only([self.register.find_node(name=self._init_params['exec_proc_name']).id])
-    if self._init_params['exec_only_list']:
-      self.exec_only(self._init_params['exec_only_list'])
-    if self._init_params['exec_disable_list']:
-      self.exec_disable(self._init_params['exec_disable_list'])
-    if self._init_params['exec_from_id'] is not None:
-      self.exec_from(self._init_params['exec_from_id'])
-    if self._init_params['exec_to_id'] is not None:
-      self.exec_to(self._init_params['exec_to_id'])
+    if self.config['exec_proc_name']:
+      self.exec_only([self.register.find_node(name=self.config['exec_proc_name']).id])
+    if self.config['exec_only_list']:
+      self.exec_only(self.config['exec_only_list'])
+    if self.config['exec_disable_list']:
+      self.exec_disable(self.config['exec_disable_list'])
+    if self.config['exec_from_id'] is not None:
+      self.exec_from(self.config['exec_from_id'])
+    if self.config['exec_to_id'] is not None:
+      self.exec_to(self.config['exec_to_id'])
   
   def execute(self):
     return self.run()
   def run(self):
     self.prepare()
     
-    # App lifecycle - RESTART
-    if self._init_params['restart']:
-      if self._on_restart_func: self._on_restart_func()
-    # App lifecycle - CREATE
-    else:
-      if self._on_create_func: self._on_create_func()
-    
-    # App lifecycle - START
-    if self._on_start_func:
-      self._on_start_func()
-    
     self.config['app_start_time'] = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
     
     # Prepare engine
@@ -210,25 +184,15 @@ class PyRunner:
     
     emit_notification = True
     
-    # App lifecycle - SUCCESS
     if retcode == 0:
-      if self._on_success_func:
-        self._on_success_func()
       if not self.config['email_on_success']:
         print('Skipping Email Notification: Property "email_on_success" is set to FALSE.')
         emit_notification = False
-    # App lifecycle - FAIL (<0 is for ABORT or other interrupt)
     elif retcode > 0:
-      if self._on_fail_func:
-        self._on_fail_func()
       if not self.config['email_on_fail']:
         print('Skipping Email Notification: Property "email_on_fail" is set to FALSE.')
         emit_notification = False
     
-    # App lifecycle - DESTROY
-    if self._on_destroy_func:
-      self._on_destroy_func()
-    
     if emit_notification:
       self.notification.emit_notification(self.config, self.register)
     
@@ -312,11 +276,12 @@ class PyRunner:
     
     return zip_file
   
-  def save_state(self, suppress_output=False):
+  def save_state(self, suppress_output=False, only_ctllog=False):
     if not suppress_output:
       print('Saving Execution Graph File to: {}'.format(self.config.ctllog_file))
-
+    
     self.serde_obj.save_to_file(self.config.ctllog_file, self.register)
+    if only_ctllog: return
     
     try:
       
@@ -371,13 +336,6 @@ class PyRunner:
       return False
     return True
   
-  # NodeRegister wiring
-  def add_node(self, **kwargs)    : return self.register.add_node(**kwargs)
-  def exec_only(self, id_list)    : return self.register.exec_only(id_list)
-  def exec_to(self, id)           : return self.register.exec_to(id)
-  def exec_from(self, id)         : return self.register.exec_from(id)
-  def exec_disable(self, id_list) : return self.register.exec_disable(id_list)
-  
   def parse_args(self):
     abort = False
     
@@ -401,23 +359,23 @@ class PyRunner:
     
     for opt, arg in opts:
       if opt == '-c':
-        self._init_params['config_file'] = arg
+        self.config['config_file'] = arg
       elif opt == '-l':
-        self._init_params['proc_file'] = arg
+        self.config['proc_file'] = arg
       elif opt in ['-d', '--debug']:
         self.config['debug'] = True
       elif opt in ['-n', '--max-procs']:
         self.config['max_procs'] = int(arg)
       elif opt in ['-r', '--restart']:
-        self._init_params['restart'] = True
+        self.config['restart'] = True
       elif opt in ['-x', '--exec-only']:
-        self._init_params['exec_only_list'] = [ int(id) for id in arg.split(',') ]
+        self.config['exec_only_list'] = [ int(id) for id in arg.split(',') ]
       elif opt in ['-N', '--norun']:
-        self._init_params['exec_disable_list'] = [ int(id) for id in arg.split(',') ]
+        self.config['exec_disable_list'] = [ int(id) for id in arg.split(',') ]
       elif opt in ['-D', '--from', '--descendents']:
-        self._init_params['exec_from_id'] = int(arg)
+        self.config['exec_from_id'] = int(arg)
       elif opt in ['-A', '--to', '--ancestors']:
-        self._init_params['exec_to_id'] = int(arg)
+        self.config['exec_to_id'] = int(arg)
       elif opt in ['-e', '--email']:
         self.config['email'] = arg
       elif opt in ['--ef', '--email-on-fail']:
@@ -429,7 +387,7 @@ class PyRunner:
         os.environ[parts[0]] = parts[1]
       elif opt == '--cvar':
         parts = arg.split('=')
-        self._init_params['cvar_list'].append((parts[0], parts[1]))
+        self.config['cvar_list'].append((parts[0], parts[1]))
       elif opt == '--nozip':
         self.config['nozip'] = True
       elif opt == '--dump-logs':
@@ -443,9 +401,9 @@ class PyRunner:
       elif opt in ['--preserve-context']:
         self.preserve_context = True
       elif opt in ['--allow-duplicate-jobs']:
-        self._init_params['allow_duplicate_jobs'] = True
+        self.config['allow_duplicate_jobs'] = True
       elif opt in ['--exec-proc-name']:
-        self._init_params['exec_proc_name'] = arg
+        self.config['exec_proc_name'] = arg
       elif opt == '--abort':
         abort = True
       elif opt in ['--serde']:
@@ -464,9 +422,9 @@ class PyRunner:
     
     # We need to check for and source the app_profile/config file ASAP,
     # but only after --env vars are processed
-    if not self._init_params.get('config_file'):
+    if not self.config['config_file']:
       raise RuntimeError('Config file (app_profile) has not been provided')
-    self.config.source_config_file(self._init_params['config_file'])
+    self.config.source_config_file(self.config['config_file'])
     
     if abort:
       print('Submitting ABORT signal to running job for: {}'.format(self.config['app_name']))
@@ -474,8 +432,8 @@ class PyRunner:
       sys.exit(0)
     
     # Check if restart is possible (ctllog/ctx files exist)
-    if self._init_params['restart'] and not self.is_restartable():
-      self._init_params['restart'] = False
+    if self.config['restart'] and not self.is_restartable():
+      self.config['restart'] = False
   
   def show_help(self):
     print("Required:")
